require 'github/markup'
require 'haml'
require 'sinatra'

module Qwiki
  # The Sinatra app. Run it like this:
  #   Qwiki::App.run!
  class App < Sinatra::Base
    enable :inline_templates

    get "*" do
      if File.directory?(request_path)
        index(request_path)
      elsif File.file?(request_path)
        GitHub::Markup.render(request_path)
      else
        raise Sinatra::NotFound
      end
    end

    private

    # Returns the full path of the file, given a path relative to the base path.
    def full_path(relative)
      # use PWD as base; later this could be a command-line option
      File.expand_path(File.join(Dir.pwd, relative))
    end

    # Returns a path relative to the base path, given the full path. This is the inverse of
    # full_path.
    def relative_path(path)
      path = File.expand_path(path)
      root = full_path("")
      if path.size >= root.size && path[0...root.size] == root
        path[0...root.size] = ""
        path = "/" if path.size == 0
        path
      end
    end

    # File path specified by the current request URL.
    def request_path
      @request_path ||= full_path(request.path)
    end

    # Renders an index page for the specified directory.
    def index(path)
      @entries = []
      Dir.entries(path).each do |entry|
        relative_path = relative_path(File.join(path, entry))
        if entry != "." && relative_path
          @entries << {:name => entry, :href => relative_path}
        end
      end
      @path = path
      haml :index
    end
  end
end

Qwiki::App.run!

__END__

@@ index
%h2= "Index of #{@path}"
%ul
  - @entries.each do |entry|
    %li
      %a{:href => entry[:href]}= entry[:name]

